@@ -10,7 +10,7 @@
      WWW: http://math.umh.ac.be/an/software/
 
    This library is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2.1 or
+   it under the terms of the GNU Lesser General Public License version 2.1 or
    later as published by the Free Software Foundation, with the special
    exception on linking described in the file LICENSE.
 
@@ -112,7 +112,7 @@ let of_in_obj ?(separator=',') ?(excel_tricks=true) in_chan = {
   excel_tricks = excel_tricks;
 }
 
-let of_channel ?separator ? excel_tricks fh =
+let of_channel ?separator ?excel_tricks fh =
   of_in_obj ?separator ?excel_tricks
     (object
        val fh = fh
@@ -173,6 +173,7 @@ end
  *)
 
 let is_space c = c = ' ' || c = '\t' (* See documentation *)
+let is_real_space c = c = ' ' (* when separator = '\t' *)
 
 (* Given a buffer, returns its content stripped of *final* whitespace. *)
 let strip_contents buf =
@@ -230,29 +231,28 @@ let add_unquoted_field ic =
     ic.record <- strip_contents ic.current_field :: ic.record;
     false
 
-(* Quoted field.  Read till a closing quote, a newline, or the end
-   of the file and decode the field.  Skip the next delimiter or
-   newline.  @return [true] if more fields follow, [false] if the
-   record is complete. *)
+(* Quoted field closed.  Read past a separator or a newline and decode
+   the field or raise [End_of_file].  @return [true] if more fields
+   follow, [false] if the record is complete. *)
 let rec seek_quoted_separator ic field_no =
   fill_in_buf ic; (* or raise End_of_file *)
   let c = String.unsafe_get ic.in_buf ic.in0 in
   ic.in0 <- ic.in0 + 1;
-  if is_space c then seek_quoted_separator ic field_no (* skip space *)
-  else if c = ic.separator || c = '\n' || c = '\r' then (
+  if c = ic.separator || c = '\n' || c = '\r' then (
     ic.record <- Buffer.contents ic.current_field :: ic.record;
     if c = '\r' then (skip_CR ic; false) else (c = ic.separator)
   )
+  else if is_space c then seek_quoted_separator ic field_no (* skip space *)
   else raise(Failure(ic.record_n, field_no,
                      "Non-space char after closing the quoted field"))
 
-let rec examine ic field_no after_quote i =
+let rec examine_quoted_field ic field_no after_quote i =
   if i >= ic.in1 then (
     (* End of field not found, need to look at the next chunk *)
     Buffer.add_substring ic.current_field ic.in_buf ic.in0 (i - ic.in0);
     ic.in0 <- i;
     fill_in_buf ic; (* or raise End_of_file *)
-    examine ic field_no after_quote 0
+    examine_quoted_field ic field_no after_quote 0
   )
   else
     let c = String.unsafe_get ic.in_buf i in
@@ -260,18 +260,18 @@ let rec examine ic field_no after_quote i =
       if c = '\"' then (
         after_quote := false;
         (* [c] is kept so a quote will be included in the field *)
-        examine ic field_no after_quote (i+1)
+        examine_quoted_field ic field_no after_quote (i+1)
       )
       else if c = ic.separator || is_space c || c = '\n' || c = '\r' then (
         seek_quoted_separator ic field_no (* field already saved; in0=i;
-                                         after_quote=true *)
+                                             after_quote=true *)
       )
       else if ic.excel_tricks && c = '0' then (
         (* Supposedly, '"' '0' means ASCII NULL *)
         after_quote := false;
         Buffer.add_char ic.current_field '\000';
         ic.in0 <- i + 1; (* skip the '0' *)
-        examine ic field_no after_quote (i+1)
+        examine_quoted_field ic field_no after_quote (i+1)
       )
       else raise(Failure(ic.record_n, field_no, "Bad '\"' in quoted field"))
     )
@@ -280,13 +280,13 @@ let rec examine ic field_no after_quote i =
       (* Save the field so far, without the quote *)
       Buffer.add_substring ic.current_field ic.in_buf ic.in0 (i - ic.in0);
       ic.in0 <- i + 1; (* skip the quote *)
-      examine ic field_no after_quote ic.in0
+      examine_quoted_field ic field_no after_quote ic.in0
     )
-    else examine ic field_no after_quote (i+1)
+    else examine_quoted_field ic field_no after_quote (i+1)
 
 let add_quoted_field ic field_no =
   let after_quote = ref false in (* preserved through exn *)
-  try examine ic field_no after_quote ic.in0
+  try examine_quoted_field ic field_no after_quote ic.in0
   with End_of_file ->
     (* Add the field even if not closed well *)
     ic.record <- Buffer.contents ic.current_field :: ic.record;
@@ -296,6 +296,7 @@ let add_quoted_field ic field_no =
 
 
 let skip_spaces ic =
+  let is_space = if ic.separator = '\t' then is_real_space else is_space in
   (* Skip spaces: after this [in0] is a non-space char. *)
   while ic.in0 < ic.in1 && is_space(String.unsafe_get ic.in_buf ic.in0) do
     ic.in0 <- ic.in0 + 1
@@ -318,7 +319,7 @@ let add_next_field ic field_no =
   Buffer.clear ic.current_field;
   try
     skip_spaces ic;
-    (* Now, in0 < in1 or raise End_of_file was raised *)
+    (* Now, in0 < in1 or End_of_file was raised *)
     let c = String.unsafe_get ic.in_buf ic.in0 in
     if c = '\"' then (
       ic.in0 <- ic.in0 + 1;
